﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ArcGIS.Core.CIM;
using ArcGIS.Core.Data;
using ArcGIS.Core.Geometry;
using ArcGIS.Desktop.Catalog;
using ArcGIS.Desktop.Core;
using ArcGIS.Desktop.Editing;
using ArcGIS.Desktop.Extensions;
using ArcGIS.Desktop.Framework;
using ArcGIS.Desktop.Framework.Contracts;
using ArcGIS.Desktop.Framework.Dialogs;
using ArcGIS.Desktop.Framework.Threading.Tasks;
using ArcGIS.Desktop.Mapping;

namespace GeoNet259853
{
	internal class CreateServiceConnection : MapTool
	{
		private FeatureLayer waterMainLayer = null;
		private FeatureLayer waterServiceLayer = null;
		private FeatureLayer waterFittingsLayer = null;
		private FeatureLayer controlValveLayer = null;

		private string waterMainName = @"GISWRKS1.WORKS.WaterMain";
		private string waterServiceName = @"GISWRKS1.WORKS.WaterService";
		private string waterFittingsName = @"GISWRKS1.WORKS.WAT_Fitting";
		private string controlValveName = @"GISWRKS1.WORKS.WAT_ControlValve";

		public CreateServiceConnection()
		{
			IsSketchTool = true;
			UseSnapping = true;
			SketchType = SketchGeometryType.Line;
			SketchOutputMode = SketchOutputMode.Map;
		}

		protected override Task OnToolActivateAsync(bool active)
		{
			waterMainLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(waterMainName, StringComparison.OrdinalIgnoreCase)).FirstOrDefault();
			if (waterMainLayer != null)
			{
				Snapping.IsEnabled = true;
				//var lstSnapModes = new SnapMode[] { SnapMode.Intersection, SnapMode.Edge, SnapMode.Vertex };
				//Snapping.SetSnapModes(lstSnapModes);
				Snapping.SetLayerSnapModes(waterMainLayer, SnapMode.Vertex, true);
				Snapping.SetLayerSnapModes(waterMainLayer, SnapMode.Edge, true);
				Snapping.SetLayerSnapModes(waterMainLayer, SnapMode.Intersection, true);
			}
			waterServiceLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(waterServiceName, StringComparison.OrdinalIgnoreCase)).FirstOrDefault();
			if (waterServiceLayer != null)
			{
				Snapping.IsEnabled = true;
				//var lstSnapModes = new SnapMode[] { SnapMode.Intersection, SnapMode.Edge, SnapMode.Vertex };
				//Snapping.SetSnapModes(lstSnapModes);
				Snapping.SetLayerSnapModes(waterServiceLayer, SnapMode.Vertex, true);
				Snapping.SetLayerSnapModes(waterServiceLayer, SnapMode.Edge, true);
				Snapping.SetLayerSnapModes(waterServiceLayer, SnapMode.Intersection, true);
			}
			waterFittingsLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(waterFittingsName, StringComparison.OrdinalIgnoreCase)).FirstOrDefault();
			if (waterFittingsLayer != null)
			{
				Snapping.IsEnabled = true;
				//var lstSnapModes = new SnapMode[] { SnapMode.Intersection, SnapMode.Edge, SnapMode.Vertex };
				//Snapping.SetSnapModes(lstSnapModes);
				Snapping.SetLayerSnapModes(waterFittingsLayer, SnapMode.Point, true);
			}
			controlValveLayer = MapView.Active.Map.GetLayersAsFlattenedList().OfType<FeatureLayer>().Where(fl => fl.Name.Equals(controlValveName, StringComparison.OrdinalIgnoreCase)).FirstOrDefault();
			if (controlValveLayer != null)
			{
				Snapping.IsEnabled = true;
				//var lstSnapModes = new SnapMode[] { SnapMode.Intersection, SnapMode.Edge, SnapMode.Vertex };
				//Snapping.SetSnapModes(lstSnapModes);
				Snapping.SetLayerSnapModes(controlValveLayer, SnapMode.Point, true);
			}
			return base.OnToolActivateAsync(active);
		}

		protected override Task<bool> OnSketchCompleteAsync(Geometry geometry)
		{
			//use the sketch geometry to create the connectionPoint and plugPoint
			var pointCollection = ((Multipart)geometry).Points;
			MapPoint connectionPoint = pointCollection[0];
			MapPoint plugPoint = pointCollection[pointCollection.Count - 1];
			MapPoint valvePoint = pointCollection[1];

			//start creating the features
			QueuedTask.Run(() =>
			{
				if (IntersectsLayer(connectionPoint, waterMainLayer) == false)
				{
					ArcGIS.Desktop.Framework.Dialogs.MessageBox.Show(connectionPoint.X.ToString() + ", " + connectionPoint.Y.ToString(), "Error");
					ArcGIS.Desktop.Framework.Dialogs.MessageBox.Show("The starting point must intersect the WaterMain or Water Service layer.", "Error");
					return;
				}

				//start a new edit operation and name it for the 'Undo' stack
				var editOp = new EditOperation();
				editOp.Name = "Create new ICI Connection";

				//set the attributes for the connectionPoint and create the point in the Fittings layer
				var connectionAttributes = new Dictionary<string, object>
								{
										{"SUBTYPECD", 6 },
										{"DIAMETER1", 200 },
										{"Shape", connectionPoint.Clone() }
								};

				editOp.Create(waterFittingsLayer, connectionAttributes);

				//set the attributes for the valve and create the point in the Control Valve layer
				var valveAttributes = new Dictionary<string, object>
								{
										{"SUBTYPECD", 2 },
										{"Shape", valvePoint.Clone() },
										{"DIAMETER", 200 }
								};

				editOp.Create(controlValveLayer, valveAttributes);

				//set the attributes for the plug and create the point in the Fittings layer
				var plugAttributes = new Dictionary<string, object>
								{
										{"SUBTYPECD", 4 },
										{"DIAMETER1", 200 },
										{"Shape", plugPoint.Clone() }
								};

				editOp.Create(waterFittingsLayer, plugAttributes);

				//set the attributes for the connection line and create the line in the Connections layer
				var lineAttributes = new Dictionary<string, object>
								{
										{"SUBTYPECD", 1 },
										{"Shape", geometry.Clone() },
										{"DIAMETER", 200 },
										{"LENGTH", GeometryEngine.Instance.Length(geometry).ToString()}
								};

				editOp.Create(waterServiceLayer, lineAttributes);

				//show an error message if not successful
				var result = editOp.Execute();
				if (result != true || editOp.IsSucceeded != true)
					throw new Exception($@"Edit failed: {editOp.ErrorMessage}");

			});

			return base.OnSketchCompleteAsync(geometry);
		}

		public static bool IntersectsLayer(MapPoint point, FeatureLayer layer)
		{
			try
			{
				SpatialQueryFilter spatialFilter = new SpatialQueryFilter();
				spatialFilter.FilterGeometry = point;
				spatialFilter.SpatialRelationship = SpatialRelationship.Intersects;

				RowCursor theCursor = layer.Search(spatialFilter);
				Feature feature;
				while (theCursor.MoveNext())
				{
					using (feature = (Feature)theCursor.Current)
					{
						return true;
					}
				}
			}
			catch (Exception ex)
			{
				ArcGIS.Desktop.Framework.Dialogs.MessageBox.Show(ex.ToString());
			}

			return false;
		}
	}
}
